<?php

namespace Makecommerce\Ecommerce\Block\System\Config;

use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Button as WidgetButton;
use Magento\Backend\Model\Url;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Framework\DataObject;
use Magento\Framework\Phrase;
use Makecommerce\Ecommerce\Helper\Data;

class Button extends Field
{
    /**
     * @var string
     */
    protected $_template = 'Makecommerce_Ecommerce::system/config/button.phtml';

    /**
     * @var Data $helper
     */
    protected $helper;

    /**
     * Button constructor.
     * @param Context $context
     * @param Url $url
     * @param Data $helper
     * @param array $data
     */
    public function __construct(
        Context $context,
        Url $url,
        Data $helper,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->_url = $url;
        $this->helper = $helper;
    }

    /**
     * Get html
     *
     * @param AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(AbstractElement $element): string
    {
        return $this->_toHtml();
    }

    /**
     * Get update url
     *
     * @return string
     */
    public function getUpdateUrl()
    {
        return $this->_url->getUrl('makecommerce/configuration/update');
    }

    /**
     * Get message title
     *
     * @return Phrase
     */
    public function getMessageTitle()
    {
        return __('Update Message');
    }

    /**
     * Get success message
     *
     * @return Phrase
     */
    public function getSuccessMessage()
    {
        return __('Payment methods were successfully updated.');
    }

    /**
     * Get failure message
     *
     * @return Phrase
     */
    public function getFailureMessage()
    {
        return __('There was an error updating payment methods.');
    }

    /**
     * Get html for the button
     *
     * @return string
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock(
            WidgetButton::class
        )->setData(
            [
                'id'      => 'custom_button',
                'label'   => __('Update'),
                'onclick' => 'updateLinks()',
            ]
        );

        return $button->toHtml();
    }

    /**
     * Gets current selected scope data
     *
     * @return DataObject
     */
    public function getScopeData()
    {
        return $this->helper->getScopeData();
    }
}
