<?php

namespace Makecommerce\Ecommerce\Block\System\Config;

use Magento\Backend\Block\Template\Context;
use Magento\Backend\Model\Url;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Phrase;

class LocationsButton extends Field
{
    /**
     * @var string
     */
    protected $_template = 'Makecommerce_Ecommerce::system/config/locations_button.phtml';

    /**
     * @var Url
     */
    protected $url;

    /**
     * LocationsButton constructor.
     * 
     * @param Context $context
     * @param Url $url
     * @param array $data
     */
    public function __construct(
        Context $context,
        Url $url,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->url = $url;
    }

    /**
     * @param AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Creates custom button html
     *
     * @return string
     * @throws LocalizedException
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock(
            \Magento\Backend\Block\Widget\Button::class
        )->setData(
            [
                'id' => 'mc_update_locations_btn',
                'label' => __('Update'),
                'onclick' => 'updateLocations(this)'
            ]
        );
        return $button->toHtml();
    }

    /**
     * Returns popup title
     *
     * @return Phrase
     */
    public function getMessageTitle()
    {
        return __('Update Message');
    }

    /**
     * Returns success message
     *
     * @return Phrase
     */
    public function getSuccessMessage()
    {
        return __('Shipping method\'s locations were successfully updated.');
    }

    /**
     * Returns error message
     *
     * @return Phrase
     */
    public function getErrorMessage()
    {
        return __('An error occurred while updating shipping method\'s locations.');
    }

    /**
     * Returns update controller's url
     *
     * @return string|null
     */
    public function getUpdateUrl()
    {
        return $this->url->getUrl('makecommerce/configuration/locations');
    }
}
